unless Kernel.respond_to?(:require_relative)
  module Kernel
    def require_relative(path)
      require File.join(File.dirname(caller[0]), path.to_str)
    end
  end
end

require_relative '../../provider/winrm'

Puppet::Type.type(:sc_virtual_machine).provide(:default, :parent => Puppet::Provider::Winrm) do

  def exists?
    filter = '| Select Name'
    result = vmm(exist_cmd+filter)
    result and result.include?(resource[:name])
  end

  def create
    if resource[:template]
      get_template = "$Template = Get-SCVMTemplate -VMMServer localhost | where { $_.Name -eq #{quote(resource[:template])} };"
      unless resource[:vm_host] 
        nodes = PSON.parse(vmm(cluster_cmd + " | Where-Object -Property Name -Like #{quote(resource[:vm_cluster],:double_star)} | Select Nodes | ForEach-Object { echo $_.nodes} | Select name | ConvertTo-Json"))
        case nodes
        when ::Array
          resource[:vm_host] = nodes.collect{|k| k["Name"]}.sample
        when ::Hash
          resource[:vm_host] = nodes['Name']
        end
      end
      get_vmhost = "$VMHost = Get-SCVMHost -ComputerName #{quote(resource[:vm_host])};"
      vmm(get_template + get_vmhost + template_cmd)
    else
      Puppet.error "Not a supported VM deployment configuration."
    end
  end

  def destroy
    command = "#{exist_cmd} | Remove-SCVirtualMachine"
    vmm(command)
  end

  def cluster_cmd
    ps_cmd('Get-SCVMHostCluster',
      {
        :VMMServer => 'localhost',
      },
      :terminate => false,
    )

  end

  def exist_cmd
    ps_cmd( 'Get-SCVirtualMachine',
      {
        :VMMServer => 'localhost',
        :Name => quote(resource[:name]),
      },
      :terminate => false,
    )
  end

  def get_username
    resource[:domain_username]
  end

  def get_password
    resource[:domain_password]
  end

  def template_cmd 
    cmd = ps_cmd( 'New-SCVirtualMachine',
      {
        :Name => quote(resource[:name]),
        :Description => quote(resource[:description]),
        :VMTemplate => '$Template',
        :VMHost => '$VMHost',
        :Path => quote(resource[:path]),
        :StartAction => resource[:start_action],
        :StopAction => resource[:stop_action],
        :CPUCount => resource[:cpu_count],
        :MemoryMB => resource[:memory_mb],
        :RunAsynchronously => resource[:async],
        :Domain => resource[:domain],
        :DomainJoinCredential => resource[:domain_username] ? '$credential' : nil,
        :HighlyAvailable => resource[:highly_available],
        :ProductKey => resource[:product_key],
      },
      :terminate => false,
    )

    if resource[:domain_username]
      credential + cmd
    else
      cmd
    end
  end
end
